//////////////////////////////////////////////////////////////////////////////////////////
// Chopper_V2: Chopper target shooting game
// Author [EAZ = Elias Abou Zeid I.D:4792637]	e_abouze@ece.concordia.ca
// Date: Dec-9-2005
//////////////////////////////////////////////////////////////////////////////////////////

#include <GL/glut.h>
#include <stdio.h>
#include <math.h> 
#include "texture.h"
#include "Environment.h"
//#include "movement.h"
#include "glm.h"
#include <mmsystem.h>

//#include "NeHeGL.h"	

/******************************************** VARIABLES DECLARATIONS *********************/

//Texture related
#define NUM_OF_TEXTURES 8						// The number of textures you are going to load in this program. 
GLuint  g_Texture[NUM_OF_TEXTURES];				// Stores the texture handles as returned by OpenGL
bool bTexFlag;									// Stores whether the texture were loaded successfully or not.

Environment Game_env; 

#define NUM_OF_SOUNDS 6
Sound	g_Sound[NUM_OF_SOUNDS];
bool bMuteALL;

//////////////////////////////////////////////////////////////////////////////////////////

GLMmodel* targetModelLists[MaxTargetCount];
GLMmodel* fragmentModelLists[MaxFragmentCount];
GLMmodel* numberModelLists[10];
GLMmodel* missileModelLists[MaxMissileModelCount];

GLMmodel*  singleBuildingModel;
GLMmodel*  manyBuildingModel;



const int TargetFileCount=2;
char* singleBuildingFileName="./data/single_Building.obj";
char* manyBuildingFileName="./data/many_buildings.obj";

char* targetFileNameLists[TargetFileCount]=
{
	"./data/tank.obj",
	"./data/plane.obj"
};



GLfloat singleBuildingCoordLists[MaxSingleBuildingCount][3];


GLfloat manyBuildingCoord[3]=
{
	50, FLOOR_LEVEL, 50
};

GLfloat targetCoordLists[MaxTargetCount][3]=
{
	{0, FLOOR_LEVEL, 5},
	{-5, FLOOR_LEVEL, -15},
	{3, FLOOR_LEVEL, 35},
	{0, FLOOR_LEVEL, 10},
	{0, FLOOR_LEVEL, 15},
	{3, FLOOR_LEVEL, 20},
	{2, TargLevel+5, -15},
	{-5, TargLevel-1, 30},
	{-10, TargLevel+3, -10},
	{2, TargLevel+10, 25},
	{-5, TargLevel+1, 15},
	{-10, TargLevel+3, 10}
};

void openFire(DWORD now, int x, int y);
extern Movement movement;
extern Rocket rockets[MaxRocketCount];
extern int rocketCount;
extern int targetCount;
extern int explosionCount;
extern int explosionIndex;
extern Rocket explosions[MaxExplosionPieces];
extern TargetStruct targets[MaxTargetCount];
bool boundaryCheck(float fx, float fy, float fz, const Movement& mv=movement);
void createExplosion(float coord[3], DWORD now);
void drawExplosionPiece(int index);
void drawExplosion(DWORD now);
void loadFragments();
void loadNumbers();
void loadMissile();
void loadBuilding();
void drawBuilding();
void drawDigit(int num, int pos, int start);
void drawNumber(int number);

////////////////////////////////////////////////////////////////////////////////////////////

//float yaw, pitch, roll;	// For the FP camera.



bool bAxis;			// To show or not to show the reference axis.

bool bWire;			// To wireframe/not.

bool bSpin;			// To spin/stop spining
bool bCircular;		// To stop/start motion light 2.
bool IsStart;   // did game start?

 
/*lights */
bool bLight1;
bool bLight2;
bool bLight3;

float fovy = FOVY;		// For Perspective projections.
float fViewVol=FVIEWVOL;			// For Orthographic projections.
int H,W; //keep track of window sizing


float mainrot_spin;  // The rotation angle for the main rotors.
float tailrot_spin; // The rotation angle for the tail rotors.

float light2_circular; // The circular rotation angle for light 2.


float radius,theta,phi; //for camera motion.   
 		
//GL_Window			window;	
DWORD				tickCount;										// Used For The Tick Counter


GLdouble eqn[4] = {0.0, 1.0, 0.0, 0.0}; //clip y=0-plane
GLdouble eqn1[4] = {0.0, 0.0, -1.0, 0.2}; //clip z=0.2-palne, remove the right hand side
GLdouble eqn2[4] = {0.0, 0.0, 1.0, -0.2}; //clip y=0.2-plan, rmeove the left hand side


/********************************************NEW:Timing (Day/Night) **************************/
void NightTime(void)
{
	Game_env.day = NIGHT; //it's night  time

	glEnable(GL_LIGHTING);

	if(!bMuteALL && !g_Sound[5].mute)
	PlaySound(g_Sound[5].filename , NULL, SND_ASYNC);

	g_Sound[3].mute=false;
}

void DayTime(void)
{
	Game_env.day = DAY; //it's night  time
	Game_env.weather  = SHINY_DAY; //set day to a shiny day
	Game_env.amdif = HIGH;  //set ambiant and diffuse to high

	glDisable(GL_LIGHTING); //no lighting during day 
	if(!bMuteALL && !g_Sound[4].mute)
	PlaySound(g_Sound[4].filename, NULL, SND_ASYNC);

	g_Sound[3].mute =false;
};

void WindSound(void) //play when no rotor sound
{
	if((!bSpin || g_Sound[1].mute) && !g_Sound[3].mute && !bMuteALL)
	{	
		PlaySound(NULL,NULL,NULL); //stop previous sound
		PlaySound(g_Sound[3].filename, NULL, SND_ASYNC);
		g_Sound[3].mute=true;
	}
}

void Timer(GLfloat time) //timing to retrieve ellapsed time 
{   
	GLfloat time_ellapsed= time - Game_env.timeday;
	

    if( time_ellapsed > HDAY)
        {
			if(Game_env.day == DAY)
				NightTime();
			else if(Game_env.day == NIGHT)
				DayTime();

			Game_env.timeday = time;
			
		}

	else if(time_ellapsed > HDAY/2)
		WindSound();
}

void drawTargets(DWORD time_elapsed);

void loadTargets();	
/****************************************************************************************/
//NeHeGL library


GLfloat		rolly;

//HDC			hDC=NULL;		// Private GDI Device Context												// Rolling Clouds												// Structure Name


void Update(DWORD milliseconds)
{
	rolly -=milliseconds*0.00005f;	// Roll The Clouds
}


//start announcement
void drawAnnounce(int Texindex)
{
	glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
	//glColor4f(0.3, 0.3, 0.3, 1.0);
	glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);
							// This command is important as the current color largely decides the color of the texture rendered.
							// To keep the original texture intact we need to set the current color to WHITE.

	// Bind the texture stored at the zero index of g_Texture[]
	glBindTexture(GL_TEXTURE_2D, g_Texture[Texindex]);	

	glBegin(GL_POLYGON);
	/*
	glTexCoord2f(1.0,0.0); glVertex3f(-4.0,1.0,0.0);
	glTexCoord2f(0.0,0.0); glVertex3f(+4.0,1.0,0.0);
	glTexCoord2f(0.0,1.0); glVertex3f(+4.0,3.0,0.0);
	glTexCoord2f(1.0,1.0); glVertex3f(-4.0,3.0,0.0);
	
	*/
	glTexCoord2f(1.0,0.0); glVertex3f(-2.0, FLOOR_LEVEL+1.5, 3.0);
	glTexCoord2f(0.0,0.0); glVertex3f(+2.0, FLOOR_LEVEL+1.5, 3.0);
	glTexCoord2f(0.0,1.0); glVertex3f(+2.0, FLOOR_LEVEL+ 3.0, 3.0);
	glTexCoord2f(1.0,1.0); glVertex3f(-2.0, FLOOR_LEVEL+ 3.0, 3.0);
	glEnd();
	glDisable(GL_TEXTURE_2D);
      
}



void drawTexturedCube(GLfloat side) //draw a cube from -side/2 to side/2 
{
/*
	glPushMatrix();	// Push The Modelview Matrix
    

	for(int i = 0; i<xslices; i++)
	{
	    for(int j=0; j<yslices; j++)
        {
			GLfloat di = floor(side/xslices);
			GLfloat dj = floor(side/yslices);
			GLfloat du = di/side;
			GLfloat dv = dj/side;
			//back side at z = -side/2
			glBegin(GL_QUADS);											// Begin Drawing Quads
			glTexCoord2f(i*du,j*dv);			glVertex3f(-side/2+i*di,-side/2+j*dj,-side/2);	// buttom left
			glTexCoord2f(du+i*du,j*dv);			glVertex3f(-side/2+(i+1)*di,-side/2+j*dj,-side/2);	// buttom right
			glTexCoord2f(du+i*du,dv+j*dv);		glVertex3f(-side/2+(i+1)*di,-side/2+(j+1)*dj,-side/2);// top right
			glTexCoord2f(i*du,dv+j*dv);			glVertex3f(-side/2+(i*di),-side/2+(j+1)*dj,-side/2);// top left
			glEnd();
		
			//front side at z = +side/2
			glBegin(GL_QUADS);											// Begin Drawing Quads
			glTexCoord2f(i*du,j*dv);			glVertex3f(-side/2+i*di,-side/2+j*dj,+side/2);	// buttom left
			glTexCoord2f(du+i*du,j*dv);			glVertex3f(-side/2+(i+1)*di,-side/2+j*dj,+side/2);	// buttom right
			glTexCoord2f(du+i*du,dv+j*dv);		glVertex3f(-side/2+(i+1)*di,-side/2+(j+1)*dj,+side/2);// top right
			glTexCoord2f(i*du,dv+j*dv);			glVertex3f(-side/2+(i*di),-side/2+(j+1)*dj,+side/2);// top left
			glEnd();

			//left side at x = +side/2
			/*glBegin(GL_QUADS);											// Begin Drawing Quads
			glTexCoord2f(i*du,j*dv);			glVertex3f(side/2,-side/2+j*dj,-side/2+i*di);	// buttom left
			glTexCoord2f(du+i*du,j*dv);			glVertex3f(-side/2,-side/2+j*dj,-side/2+(i+1)*di);	// buttom right
			glTexCoord2f(du+i*du,dv+j*dv);		glVertex3f(-side/2,-side/2+(j+1)*dj,-side/2+(i+1)*di);// top right
			glTexCoord2f(i*du,dv+j*dv);			glVertex3f(-side/2,-side/2+(j+1)*dj,-side/2+(i*di));// top left
			glEnd();
		}
	}*/
	
	/*//left side at x = +side/2
			glBegin(GL_QUADS);											// Begin Drawing Quads
			glTexCoord2f(i*di,j*dj);			glVertex3f(+side/2,-side/2+j*dj,+side/2);	// buttom left
			glTexCoord2f(di+i*di,j*dj);			glVertex3f(+side/2,-side/2+j*dj,+side/2);	// buttom right
			glTexCoord2f(di+i*di,dj+j*dj);		glVertex3f(+side/2,-side/2+(j+1)*dj,+side/2);// top right
			glTexCoord2f(i*di,dj+j*dj);			glVertex3f(+side/2,-side/2+(j+1)*dj,+side/2);// top left
			glEnd();
	
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-FLOOR_SIZE/2.0f,+97.5f,-FLOOR_SIZE/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+FLOOR_SIZE/2.0f,+97.5f,-FLOOR_SIZE/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+FLOOR_SIZE/2.0f,+97.5f,+FLOOR_SIZE/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-FLOOR_SIZE/2.0f,+97.5f,+FLOOR_SIZE/2.0f);	// Bottom Right	// Bottom Right
	glEnd();
												
	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-FLOOR_SIZE/2.0,-10.0,+FLOOR_SIZE/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+FLOOR_SIZE/2.0f,-10.0f,+FLOOR_SIZE/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+FLOOR_SIZE/2.0f,+97.5,+FLOOR_SIZE/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-FLOOR_SIZE/2.0f,+97.5f,+FLOOR_SIZE/2.0f);	// Bottom Right
    glEnd();
	
	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-FLOOR_SIZE/2.0,-10.0,-FLOOR_SIZE/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(-FLOOR_SIZE/2.0f,-10.0f,+FLOOR_SIZE/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(-FLOOR_SIZE/2.0f,+97.5,+FLOOR_SIZE/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-FLOOR_SIZE/2.0f,+97.5f,-FLOOR_SIZE/2.0f);	// Bottom Right
    glEnd();

	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(+FLOOR_SIZE/2.0,-10.0,-FLOOR_SIZE/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+FLOOR_SIZE/2.0f,-10.0f,+FLOOR_SIZE/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+FLOOR_SIZE/2.0f,+97.5,+FLOOR_SIZE/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(+FLOOR_SIZE/2.0f,+97.5f,-FLOOR_SIZE/2.0f);	// Bottom Right
    glEnd();

	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-FLOOR_SIZE/2.0,-10.0f,-FLOOR_SIZE/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+FLOOR_SIZE/2.0f,-10.0f,-FLOOR_SIZE/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+FLOOR_SIZE/2.0f,-10.0f,+FLOOR_SIZE/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-FLOOR_SIZE/2.0f,-10.0f,+FLOOR_SIZE/2.0f);	// Bottom Right
    glEnd();

    glPopMatrix();*/

	glPushMatrix();												// Push The Modelview Matrix
	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0f,-10.0f,-side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0f,-10.0f,-side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,side/2 + 10.0,-side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,side/2 + 10.0,-side/2.0f);	// Bottom Right
    glEnd();
	
	glBegin(GL_QUADS);
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0f,side/2+10.0,-side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0f,side/2+10.0,-side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Right	// Bottom Right
	glEnd();
												
	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0,-10.0,+side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0f,-10.0f,+side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Right
    glEnd();
	
	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0,-10.0,-side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0f,-10.0f,+side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,side/2+10.0,-side/2.0f);	// Bottom Right
    glEnd();

	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0,-10.0,-side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0f,-10.0f,+side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,side/2+10.0,+side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,side/2+10.0,-side/2.0f);	// Bottom Right
    glEnd();

	glBegin(GL_QUADS);											// Begin Drawing Quads
		glTexCoord2f(1.0f,rolly/1.5f+1.0f); glVertex3f(-side/2.0,-10.0f,-side/2.0f);	// Top Right
		glTexCoord2f(0.0f,rolly/1.5f+1.0f); glVertex3f(+side/2.0f,-10.0f,-side/2.0f);	// Top Left
		glTexCoord2f(0.0f,rolly/1.5f+0.0f); glVertex3f(+side/2.0f,-10.0f,+side/2.0f);	// Bottom Left
		glTexCoord2f(1.0f,rolly/1.5f+0.0f); glVertex3f(-side/2.0f,-10.0f,+side/2.0f);	// Bottom Right
    glEnd();

    glPopMatrix();

}


/****************************************************************************************/
void drawGun(void)
{
	//gun-hand 
	glBegin(GL_POLYGON);
	{
		glColor4f(Game_env.amdif,Game_env.amdif,Game_env.amdif,Game_env.amdif);
		glVertex3f(0.3,-0.03,0.25);
		glVertex3f(0.45,-0.04,0.25);
		glVertex3f(0.45,-0.04,0.30);
		glVertex3f(0.3,-0.03,0.30);
	}
	glEnd();

	//gun base
	glPushMatrix();
	glTranslatef(0.45,-0.04,0.2);
	//glRotatef(-15.0,0.0,1.0,0.0);
	drawCylinder(0.04,0.04,0.15,20,20);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.445,-0.04,0.22);
	drawSphere(0.043,20,20);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0.445,-0.04,0.34);

	drawSphere(0.0445,20,20);
	glPopMatrix();

	//gun
	glPushMatrix();
	glTranslatef(0.445,-0.04,0.27);
	drawCylinder(0.01,0.005,0.25,20,20);
	glPopMatrix();
}

void drawCockpit(void)
{
  	
  
	glPushMatrix();

	glScalef(1.0,1.5,2.0);//translate by (1.0,1.0,0.0)
	glClipPlane (GL_CLIP_PLANE0, eqn); //clip plane at y=0, remove down side
	glEnable (GL_CLIP_PLANE0);
	glClipPlane (GL_CLIP_PLANE1, eqn1); //clip plane at z=0.2, remove the right hand side
	glEnable (GL_CLIP_PLANE1);
	glPushMatrix();
	glRotatef(180.0,0.0,1.0,0.0);

	glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
	glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);
							// This command is important as the current color largely decides the color of the texture rendered.
							// To keep the original texture intact we need to set the current color to WHITE.

	// Bind the texture stored at the zero index of g_Texture[]
	glBindTexture(GL_TEXTURE_2D, g_Texture[0]);		

	drawSphere(0.45, 100, 100);//draw a wire sphere with radius 0.25, with 15 divisions around z-axis and 15 divisions along z-axis. 
	glPopMatrix();
	glDisable (GL_CLIP_PLANE0);
	glDisable (GL_CLIP_PLANE1);

	//cover-down		
	glPushMatrix();
	glTranslatef(0.0,-0.01,0.0);
	glScalef(1.0,0.20,1.0);
	drawSphere(0.45, 100, 100);
	glPopMatrix();


	//left-gun
	glBindTexture(GL_TEXTURE_2D, g_Texture[0]);
	drawGun();

	//right-gun
	glPushMatrix();
	glScalef(-1.0,1.0,1.0);
	drawGun();
	glPopMatrix();

	//front window
	glDisable(GL_TEXTURE_2D);	// disable Texture Mapping
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDepthMask(GL_FALSE);
	glColor4f(1.0, 1.0,1.0,0.3);
	glClipPlane (GL_CLIP_PLANE2, eqn2); //clip plane at z=-0.2, remove the left hand side
	glEnable (GL_CLIP_PLANE2);
	glEnable (GL_CLIP_PLANE0);
	drawSphere(0.45, 30, 30);//draw a wire sphere with radius 0.45, with 30 divisions around z-axis and 30 divisions along z-axis. 
	glDisable (GL_CLIP_PLANE0);
	glDisable (GL_CLIP_PLANE2);
	glColor3f(0.4, 0.5, 0.4);
	glDisable (GL_BLEND);
	glDepthMask(GL_TRUE);


	glPopMatrix();
}

/****************************************************************************************/
void drawLegs(void)
{
  glPushAttrib(GL_COLOR_BUFFER_BIT);		
  glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
 
  glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);
  // Bind the texture stored at the zero index of g_Texture[]
  glBindTexture(GL_TEXTURE_2D, g_Texture[0]);		

	//front legs	
	glPushMatrix();
		glTranslatef(0.0,-0.09,0.0);
		glRotatef(90.0,1.0,0.0,0.0);	
 		glPushMatrix();
		glTranslatef(-0.125,0.4,0.0);			
		glRotatef(-45.0,0.0,1.0,0.0);
		drawCylinder(0.02,0.02,0.2,5.0,5.0);
		glPopMatrix();
		glPushMatrix();
		glTranslatef(0.125,0.4,0.0);
		glRotatef(45.0,0.0,1.0,0.0);
		drawCylinder(0.02,0.02,0.2,5.0,5.0);
		glPopMatrix();
	glPopMatrix();

	
	//back wheel leg
	glPushMatrix();
		glTranslatef(0.0,0.0,-1.91);
		glRotatef(90.0,1.0,0.0,0.0);
		drawCylinder(0.02,0.02,0.17,5.0,5.0);
	glPopMatrix();
	
	// back Wheel leg support
	glPushMatrix();
		glTranslatef(0.0,0.0,-1.65);
		glRotatef(45.0,1.0,0.0,0.0);
		glRotatef(90.0,1.0,0.0,0.0);
		drawCylinder(0.02,0.02,0.28,5.0,5.0);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);

	glPopAttrib();
	
}

/****************************************************************************************/
void drawFrontWheels ( bool Bwire)
{
	//front wheels:
	if(Bwire)
	{

		glPushMatrix();
			glTranslatef(-0.251,-0.30,0.4);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutWireTorus(0.04, 0.08, 10, 10);
		glPopMatrix();			 
		glPushMatrix();
			glTranslatef(0.251,-0.30,0.4);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutWireTorus(0.04, 0.08, 10, 10);
		glPopMatrix();
	}
	else
	{

		glPushMatrix();
			glTranslatef(-0.251,-0.30,0.4);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutSolidTorus(0.04, 0.08, 10, 10);
		glPopMatrix();			 
		glPushMatrix();
			glTranslatef(0.251,-0.30,0.4);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutSolidTorus(0.04, 0.08, 10, 10);
		glPopMatrix();
	}
}

/****************************************************************************************/
void drawBackWheel( bool Bwire)
{
	if(Bwire)
	{
		glPushMatrix();
			glTranslatef(0.0,-0.32,-1.9);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutWireTorus(0.06, 0.1, 10, 10);
		glPopMatrix();
	}
	else
	{
		glPushMatrix();
			glTranslatef(0.0,-0.27,-1.9);
			glRotatef(90,1.0,0.0,0.0);
			glRotatef(90,0.0,1.0,0.0);
			glutSolidTorus(0.06, 0.1, 10, 10);
		glPopMatrix();
	}
}

/****************************************************************************************/
void drawWheels(void)
{
  glPushAttrib(GL_COLOR_BUFFER_BIT);
  
  glColor4f(0.0,0.0,0.0,1.0);

  //front wheels
  drawFrontWheels(bWire);
	
  //back wheel
  drawBackWheel(bWire);

  glPopAttrib();
 
}

/****************************************************************************************/
void drawTail(void)
{
	glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
	glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);
	
	// Bind the texture stored at the zero index of g_Texture[]
	glBindTexture(GL_TEXTURE_2D, g_Texture[0]);	

	glPushMatrix();
		glTranslatef(0.0,0.0,-0.87); 
		glRotatef(180,0.0,1.0, 0.0); //rotate by 180deg. around y-axis
		glEnable (GL_CLIP_PLANE0); //clip at y=0, remove negative y part.
		drawCylinder(0.15, 0.15/4, 1.1, 30.0, 30.0);
		glDisable (GL_CLIP_PLANE0);//disable clipping
		glScalef(1.0,0.15,1.0);	
		drawCylinder(0.15, 0.15/4, 1.1, 30.0, 30.0);
	glPopMatrix();	
	glDisable(GL_TEXTURE_2D);
}

/****************************************************************************************/
void drawTailHead(void)
{
	glPushAttrib(GL_COLOR_BUFFER_BIT);			// Use GL_ALL_ATTRIB_BITS if you are not sure of the attribute 
												// bits which are going to be modified 
		glColor3f(Game_env.amdif, Game_env.amdif, Game_env.amdif);				// This steps ensures that the current color is set to white.
												// It is done to ensure that a different current color doesn't mix up with the 
												// texture's original colors. 

		// Below we draw a texture mapped square. Remember, GL_QUADS draws a 4 point polygon.
		// In order to assign a texture map to a polygon, we need to call glBindTexture().
		// This passes in the type of texture map (always use GL_TEXTURE_2D) and the index
		// into our generated texture array - g_Texture[].  If we want to use another
		// texture map, and we have multiple loaded, we just change the index into the array.

		glEnable(GL_TEXTURE_2D);
		// Bind the texture stored at the zero index of g_Texture[]
		glBindTexture(GL_TEXTURE_2D, g_Texture[0]);

		// Display a quad texture to the screen
		glBegin(GL_QUADS);

		// glTexCoord2f() takes the X and Y offset (or U and V) into the bitmap.
		// Then, the next point sent to be rendered attaches that part of the bitmap
		// to itself.  The (U, V) coordinates range from (0, 0) being the top left corner
		// of the bitmap, to (1, 1) being the bottom left corner of the bitmap.
		// You can go above 1 but it just is wrapped around back to zero and repeats the texture.
		// Try setting the 1's to 2's and see what it does, then try setting them to 0.5's.
		// The higher the number, the more instances of the texture will appear on the square,
		// Where the lower the number, it stretches the incomplete texture over the surface of the square.
		// For every vertice we need a U V coordinate, as shown below.  You might have to play
		// around with the values to make it texture correctly, otherwise it will be flipped, upside down,
		// or skewed.  It also depends on where you are looking at it.  We are looking down the -Z axis.

			// Display the top left vertice
			if(bTexFlag)
				glTexCoord2f(0.0f, 0.0f);
			glVertex3f(0.0,0.0, -1.96);

			// Display the bottom left vertice
			if(bTexFlag)
				glTexCoord2f(10.0f, 0.0f);
			glVertex3f(0.0,0.0, -1.80);

			// Display the bottom right vertice
			if(bTexFlag)
				glTexCoord2f(10.0f, 10.0f);
			glVertex3f(0.0,0.45, -1.960);

			// Display the top right vertice
			if(bTexFlag)
				glTexCoord2f(0.0f, 10.0f);
			glVertex3f(0.0,0.45, -2.03);

		glEnd();
			glDisable(GL_TEXTURE_2D);

	glPopAttrib();
}	

/****************************************************************************************/
void drawTailRotors(bool Bwire)
{	
	glPushAttrib(GL_COLOR_BUFFER_BIT);
		if(bSpin)
		{
			glEnable (GL_BLEND);
			glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glDepthMask(GL_FALSE);
			
			
			for(int i =0; i<90;i++) // motion blur alpha blending/blade acummulation 
			{
				glPushMatrix();
				glColor4f(0.0,0.0,0.0,1.0 -0.2*float(i));

				//1st rotor
					glTranslatef(0.05,0.35,-1.97);
					glRotatef(tailrot_spin + 3.0*float(i),1.0,0.0,0.0);
					glTranslatef(-0.05,-0.35,+1.97);
				glBegin(GL_POLYGON);
						glVertex3f(0.05, 0.05,-1.95);
						glVertex3f(0.05, 0.05,-1.99);
						glVertex3f(0.05, 0.65,-1.99);
						glVertex3f(0.05, 0.65,-1.95);
				glEnd();

				//2nd rotor
					glTranslatef(0.05,0.35,-1.97);
					glRotatef(tailrot_spin + 3.0*float(i),1.0,0.0,0.0);
					glTranslatef(-0.05,-0.35,+1.97);
				
				glBegin(GL_POLYGON);
						glVertex3f(0.05, 0.33,-1.67);
						glVertex3f(0.05, 0.33,-2.29);
						glVertex3f(0.05, 0.37,-2.29);
						glVertex3f(0.05, 0.37,-1.67);
				glEnd();
				glPopMatrix();

			}
			glDisable (GL_BLEND);
			glDepthMask(GL_TRUE);

		}
		else
		{
			//1st rotor
			glPushMatrix();
				glTranslatef(0.05,0.35,-1.97);
				glRotatef(tailrot_spin,1.0,0.0,0.0);
				glTranslatef(-0.05,-0.35,+1.97);
			glBegin(GL_POLYGON);
					glVertex3f(0.05, 0.05,-1.95);
					glVertex3f(0.05, 0.05,-1.99);
					glVertex3f(0.05, 0.65,-1.99);
					glVertex3f(0.05, 0.65,-1.95);
			glEnd();
		
			glBegin(GL_POLYGON);
					glVertex3f(0.05, 0.33,-1.67);
					glVertex3f(0.05, 0.33,-2.29);
					glVertex3f(0.05, 0.37,-2.29);
					glVertex3f(0.05, 0.37,-1.67);
			glEnd();
			glPopMatrix();
		}
}

/****************************************************************************************/
void drawTailRotorsBase(void)
{
	//draw back head
	glPushAttrib(GL_COLOR_BUFFER_BIT);
	glColor4f(Game_env.amdif,Game_env.amdif,Game_env.amdif,1.0);

	glEnable(GL_TEXTURE_2D);
		// Bind the texture stored at the zero index of g_Texture[]
		glBindTexture(GL_TEXTURE_2D, g_Texture[0]);
	 glPushMatrix();
		glTranslatef(0.0,0.35,-1.97); 
		glRotatef(90,0.0,1.0, 0.0); //rotate by 90deg. around y-axis
		drawCylinder(0.03, 0.001, 0.10, 5.0, 5.0); //draw a cylinder with base radius 0.3, up radius 0.001, height 0.1, with 5 divisions around z-axis and 5 divisions along z-axis.
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);
	glPopAttrib();

}


/****************************************************************************************/
void drawChopperEnd(void)
{
	//Draw tail
	drawTail();

	//Draw tail head
	drawTailHead();	

}


/****************************************************************************************/
void drawMainRotors(bool Bwire)
{	
		glPushAttrib(GL_COLOR_BUFFER_BIT);
		if(bSpin)
		{
			glEnable (GL_BLEND);
			glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glDepthMask(GL_FALSE);
			 
			glPushMatrix();
			glRotatef(mainrot_spin,0.0,1.0,0.0);	
			for (int i=0; i<90; i++) //motion blur effect: Alpha Blending/blades accumulation
				{	
					glColor4f(0.0, 0.0,0.0,1.0 - (0.2*i)); 
			   		glPushMatrix();
						glRotatef(3.0*float(i),0.0,1.0,0.0);
						glTranslatef(0.0,0.71,0.0); //translate by (0.0,0.71,0.0)
						glPushMatrix();
						glScalef(0.33, 0.00,7);
						
						if(Bwire)
							glutWireCube(0.25);
						else  glutSolidCube(0.25);

						glPopMatrix();
						glRotatef(90.0 + 3.0*float(i),0.0,1.0,0.0);
						glScalef(0.33, 0.00,7);
						
						if(Bwire)
							glutWireCube(0.25);
						else  glutSolidCube(0.25);

					glPopMatrix();
				}
		   glPopMatrix();
			
		   glDisable (GL_BLEND);
		   glDepthMask(GL_TRUE);

		   glPopAttrib();
		}
		else
		{
			glPushAttrib(GL_COLOR_BUFFER_BIT);
			glColor4f(0.0,0.0,0.0,1.0);

			glPushMatrix();
			glTranslatef(0.0,0.71,0.0); //translate by (0.0,0.71,0.0)
			glPushMatrix();
			glScalef(0.3, 0.00,7);

			if(Bwire)
				glutWireCube(0.25);
			else  glutSolidCube(0.25);
			
			glPopMatrix();
			glRotatef(90.0,0.0,1.0,0.0);
			glScalef(0.3, 0.00,7);
		
			if(Bwire)
				glutWireCube(0.25);
			else  glutSolidCube(0.25);
						
			glPopMatrix();
		}
	glPopAttrib();
}


/****************************************************************************************/
void drawHead(void)
{  
  glPushAttrib(GL_COLOR_BUFFER_BIT);		
  glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
  
  glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);

  // Bind the texture stored at the zero index of g_Texture[]
  glBindTexture(GL_TEXTURE_2D, g_Texture[0]);		

	glPushMatrix();
		glTranslatef(0.0,0.65,0.0); //translate by (0.0,0.65,0.0)
		glRotatef(-90,1.0,0.0, 0.0); //rotate by -90deg. around x-axis
		drawCylinder(0.03, 0.0, 0.15, 5.0, 5.0); 
	glPopMatrix();
	
  glDisable(GL_TEXTURE_2D); //disable texture mapping
  glPopAttrib();


};


void shinyLight()
{	
	if(Game_env.IsShiny)
	{

		GLfloat matSpec[4] = {1.0, 1.0, 1.0, 1.0};
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, matSpec);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 100.0);
		Game_env.amdif=LOW; //put a low diffuse and ambiant to emphasis specular(shiny, presence)
	}
	else
    {
		Game_env.amdif=HIGH;
		GLfloat matSpec[4] = {0.0, 0.0, 0.0, 0.0};
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, matSpec);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 00.0);
	}
	
	
}
/****************************************************************************************/

/****************************************************************************************/
void drawChopper(void)
{
		
	glPushAttrib(GL_ALL_ATTRIB_BITS);
		
		glPushMatrix();
		//glTranslatef(0.0,HecLevel,0.55);
		//step1: draw chopper skeleton
		/***********************/
		drawChopperEnd(); //draw back part

		//step2: draw accescories
		/***********************/
		drawHead(); //draw head 
		
		drawTailRotorsBase();//Draw rotors base
		drawLegs(); //draw legs
		drawWheels(); //draw wheels
		
		//step3: draw blended parts
		/**************************/
		drawCockpit(); //draw cockpit
		drawMainRotors(bWire);	//Draw main rotors
		drawTailRotors(bWire); //Draw tail rotors
		glPopMatrix();
	glPopAttrib();
}


/************* NEW *****************/

void Draw3DSGrid(int w, int h)
//	 Draws a tesselated 3D grid. Can control its tesselation as an input parameter.
//   Finer tesselation, better lighting, but slower performance.
{
  int i, j;
  float dw = FLOOR_SIZE / w;
  float dh = FLOOR_SIZE / h;
  GLfloat du = dw/FLOOR_SIZE;
  GLfloat dv = dh/FLOOR_SIZE;

  glColor3f(Game_env.amdif,Game_env.amdif,Game_env.amdif);
  glPushMatrix();
  glTranslatef(-FLOOR_SIZE/2, -2.5, -FLOOR_SIZE/2);
  glNormal3f(0.0, 1.0, 0.0);
  
   glEnable(GL_TEXTURE_2D);
		// Bind the texture stored at the zero index of g_Texture[]
	glBindTexture(GL_TEXTURE_2D, g_Texture[1]);

  for (j = 0; j < h; ++j) {
    glBegin(GL_TRIANGLE_STRIP);
    for (i = 0; i <= w; ++i) {
	    glTexCoord2f(du * i, dv * (j + 1));
		glVertex3f(dw * i, 0.0, dh * (j + 1));
	    //glVertex3f(dw * i, 0*sin(i*PI/180.0), dh * (j + 1));
		
		glTexCoord2f(du * i, dv * j);
		//glVertex3f(dw * i, 0*sin(i*PI/180.0), dh * j);
		glVertex3f(dw * i, 0.0, dh * j);
    }
    glEnd();
  }
  glPopMatrix();
  glDisable(GL_TEXTURE_2D);
}
/****************************************************************************************/
void drawChopperBase()
{
	glPushAttrib(GL_COLOR_BUFFER_BIT);
    
	glEnable(GL_TEXTURE_2D);	// enable Texture Mapping
	glColor4f(Game_env.amdif, Game_env.amdif, Game_env.amdif, 1.0);
							// This command is important as the current color largely decides the color of the texture rendered.
							// To keep the original texture intact we need to set the current color to WHITE.

	 // Bind the texture stored at the index of g_Texture[]
	 glBindTexture(GL_TEXTURE_2D, g_Texture[3]);		

	//draw square
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f,0.0f); glVertex3f(2.0,-2.459,-2.0);
		glTexCoord2f(1.0f,0.0f); glVertex3f(-2.0,-2.459,-2.0);
		glTexCoord2f(1.0f,1.0f); glVertex3f(-2.0,-2.459,+2.0);
		glTexCoord2f(0.0f,1.0f);  glVertex3f(+2.0,-2.459,+2.0);
    glEnd();
	
   glDisable(GL_TEXTURE_2D);
}


/****************************************************************************************/
void drawChopperBases()
{
	glColor3f(1.0,0.2,0.2);
	//drawBaseAt(2.0);
	//drawBaseAt(-2.0);
	drawChopperBase();

}
/****************************************************************************************/
void drawSkyballon(void)	
{
	glColor3f(1.0,1.0,0.0);
	glPushMatrix();
	glTranslatef(0.0,0.2,20.0);
	glScalef(0.6,0.8,0.7);
	drawSphere(0.20,10.0,10.0);
	glColor3f(0.0,0.0,0.0);
	glBegin(GL_LINES);
		glVertex3f(0.0,0.0,0.0);
		glVertex3f(0.0,-1,0.0);
	glEnd();
	glPopMatrix();
	
	glColor3f(0.0,1.0,0.0);
	glPushMatrix();
	glTranslatef(0.25,0.2,20.0);
	glScalef(0.6,0.8,0.7);
	drawSphere(0.20,10.0,10.0);
	glColor3f(0.0,0.0,0.0);
	glBegin(GL_LINES);
		glVertex3f(0.15,0.0,0.0);
		glVertex3f(-0.6,-1,0.0);
	glEnd();
	glPopMatrix();

	glColor3f(0.0,0.0,1.0);
	glPushMatrix();
	glTranslatef(-0.25,0.2,20.0);
	glScalef(0.6,0.8,0.7);
	drawSphere(0.20,10.0,10.0);
	glColor3f(0.0,0.0,0.0);
		glBegin(GL_LINES);
		glVertex3f(-0.15,0.0,0.0);
		glVertex3f(+0.6,-1.0,0.0);
	glEnd();
	glPopMatrix();
}
/****************************************************************************************/
void drawParaBombs()
{
	glPushMatrix();
	glTranslatef(0.0,TargLevel,0.0);
	drawParaBombAt(0.0,3.0);
	glPopMatrix();
}

/****************************************************************************************/
void drawSky()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
    if(Game_env.weather == FOGY_DAY && Game_env.day == DAY)
	glColor4f(0.5,0.5,0.5,1.0); //fogy sky color
	else glColor4f(1.0,1.0,1.0,1.0);

	if(Game_env.weather == SHINY_DAY || Game_env.day == NIGHT)
		glEnable(GL_TEXTURE_2D);	// Enable Texture Mapping, only if its is shiny day or night otherwise, fog effect

	if(Game_env.day == DAY)
	glBindTexture(GL_TEXTURE_2D, g_Texture[2]);			// Select day Sky Texture
	
	else if(Game_env.day == NIGHT)
			glBindTexture(GL_TEXTURE_2D, g_Texture[5]);	// Select night Sky Texture
	
	glPushMatrix();
	glScalef(1.0,1.0,0.75);
    drawTexturedCube(6*FLOOR_SIZE);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);	// Disable Texture Mapping

	glPopAttrib();
}
/****************************************************************************************/
void drawEnvironment(void)	
{
	Draw3DSGrid(100,100);
	drawParaBombs();
	drawGroundTrees();
	drawChopperBases();
	drawSkyballon();
	drawSky();
}


/****************************************************************************************/
void showReferenceAxis(void)
						// Draw the reference axis
{
	if(bAxis)
	{
	 glPushAttrib(GL_ALL_ATTRIB_BITS);
						// Pushes attributes like current color to attribute stack.
	 glBegin(GL_LINES);
						// X axis red
      glColor3f(1, 0, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(1, 0, 0);
						// Y axis green
      glColor3f(0, 1, 0);
      glVertex3f(0, 0, 0);
      glVertex3f(0, 2, 0);
						// Z axis blue
      glColor3f(0, 0, 2);
      glVertex3f(0, 0, 0);
      glVertex3f(0, 0, 2);
	  glEnd();
	  glPopAttrib();
						// // Pops attributes like current color from the attribute stack and sets as current.
	}
}

/****************************************************************************************/
void showMenu(void)
{
	printf("\n\n ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////");
	printf("\n OpenGL PROGRAM FOR DRAWING A HELICOPTER AND OPERATING ON OT\n");
	printf("\n OCTOBER, 15 2005 \n");
	printf("////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////\n");
	printf("\n\n Operations: ");
	printf("\n Use a/A keys to show/hide reference axis projection.");
	printf("\n\n Use P/p and N/n to switch between orthogonal and perspective view. ");
	printf("\n Use UP/DOWN arrow to move chopper forward/backward at current heading direction.");
	printf("\n Use LEFT/RIGHT arrow to move chopper at current left/right-hand-side direction.");
	printf("\n Use the f/b,keys to move third-person-view camera forward/backward.");
	printf("\n Use w/W keys to toggle between wire frame/shaded model.");
	printf("\n Use o/O or n/N key to toggle between orthographic and perspective projection.");
	printf("\n Use z/Z key to zoom in and out respectively.");
	printf("\n Use c/C key to clear viewing parameters and return to default.");
	printf("\n Use s/S to toggle between spin on/off rotors.");
	printf("\n Use 1/3 key to toggle between 1st and 3rd person view.");
	printf("\n Use y/Y key for a Yaw rotation.");
	printf("\n Use p/P key for a pitch rotation.");
	printf("\n Use i/I and k/K to move rotate the center of projection up/down about the center of gravity of chopper.");
	printf("\n Use l/L to turn on/off sound effect.");
	printf("\n Use F1 to turn on/off light 1.");
	printf("\n Use F2 to turn on/off light 2.");
	printf("\n Use 6/7 rotate third-person-view camera around chopper horizontally.");
	printf("\n Use 8/9 rotate third-person-view camera around chopper vertically.");

	printf("\n Use U/u/D/d to move chopper upward/downward vertically");
	printf("\n Click mouse at first-person-view mode for firing rockets");

	printf("\n Use F4 to toggle the material property rusty/shiny");
	printf("\n Use 2 to turn on/off motion of light 2.");
	printf("\n Use h/H to control low/high beam of light 1.");
	printf("\n Use n/M to mute/play rotor sounds.");
	printf("\n Use x or M to see this menu again.");
	printf("\n Use esc key to exit gracefully from program.");
}

/******************** NEW ****************************/

// For and light properties.

GLfloat green[] = { 0.4, 1.0, 0.4, 1.0 };
GLfloat red[] = {1.0,0.0,0.0,1.0};
GLfloat blue[]={0.0,0.0,1.0,1.0};

GLfloat lightAmbient[] = { 0.3, 1.0, 0.4, 1.0 };
		
GLfloat lightone_position[] = { 0.0, 0.0, 0.8, 1.0 };
GLfloat lightone_direction[] = { 0.0,-2, 10.0 };
GLfloat fSpotLightone = 30.0;
GLfloat fExpLightone =2.0;

GLfloat lighttwo_position[] = { 1.0, 5.0, 0.0, 1.0 };
GLfloat lighttwo_direction[] = { 0.0, -1.0, 0.0 };
GLfloat fSpotLighttwo = 15.0;

//GLfloat lightthr_position[] = { 0.0, 0.0, 0.0, 1.0 };
//GLfloat lightthr_direction[] = { 0.0, 0.0, -1.0 };
//GLfloat fSpotLightthr = 30.0;


void initLightOne(void)
{
	/* Initializtion of light 1 */
	glLightfv(GL_LIGHT0, GL_AMBIENT, red);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, red);
	glLightfv(GL_LIGHT0, GL_SPECULAR, red);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, fSpotLightone);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, fExpLightone);
	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.0);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.0);
};

void initLightTwo(void)
{
	/* Initializtion of light 2 */
	glLightfv(GL_LIGHT1, GL_AMBIENT, blue);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, blue);
	glLightfv(GL_LIGHT1, GL_SPECULAR, blue);
	glLightf(GL_LIGHT1, GL_SPOT_CUTOFF, fSpotLighttwo);
	glLightf(GL_LIGHT1, GL_SPOT_EXPONENT, 2.0);
	glLightf(GL_LIGHT1, GL_CONSTANT_ATTENUATION, 1.0);
	glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.0);
	glLightf(GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.0);
};

/*
void initLightThree(void)
{
	// Initializtion of light 3 
	glLightfv(GL_LIGHT2, GL_AMBIENT, green);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, green);
	glLightfv(GL_LIGHT2, GL_SPECULAR, green);
	glLightf(GL_LIGHT2, GL_SPOT_CUTOFF, fSpotLightthr);
	glLightf(GL_LIGHT2, GL_SPOT_EXPONENT, 2.0);
	glLightf(GL_LIGHT2, GL_CONSTANT_ATTENUATION, 1.0);
	glLightf(GL_LIGHT2, GL_LINEAR_ATTENUATION, 0.0);
	glLightf(GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.0);
};
*/

void initLights(void)
					// Initialize the common lights related data.
{
	initLightOne();
	initLightTwo();
//	initLightThree();

}

/****************************************** SOUND EFFECT RELATED ***************************/

void RotorSound(int w) //set action taken when image is idle no interaction from user.
{
	if(bSpin && !g_Sound[1].mute && !bMuteALL)
	{	
		if(Game_env.camera == THIRD_PERSON)
		{ 
		  PlaySound(g_Sound[1].filename, NULL, SND_ASYNC);
		  glutTimerFunc(g_Sound[1].duration,RotorSound, 10);
		} 
	}
	else {
		
		PlaySound(NULL, NULL, NULL); //kill rotor sound directly
	}

	
}		

void BattleSound(int w) //set this sound to play when game is not started or game over
{
	if(!IsStart && !bMuteALL)
	{	 
		  PlaySound(g_Sound[0].filename, NULL, SND_ASYNC);
		  glutTimerFunc(g_Sound[0].duration,BattleSound, 10);
	 
	}
	else
	{ 
		PlaySound(NULL,NULL,NULL);
		g_Sound[0].mute = true;
	}
	
}			
/****************************************************************************************/


void init (void)
// Initializes the gl Graphics env and the program variables.
{		
		   
   //initialization of game environment environment		
   Game_env.camera		  = THIRD_PERSON;
   Game_env.day			  = DAY;
   Game_env.projection	  = PERSPECTIVE;
   Game_env.weather		  = SHINY_DAY;
   Game_env.IsShiny		  = false; //is the material shiny
   Game_env.amdif	      = HIGH; //set diffuse and ambiant material property to high 
   Game_env.timeday       = 0.0;
   Game_env.lastTickCount = GetTickCount();

  //initializing all light to on
	bLight1 = true;
	bLight2 = true;
	bLight3 = true;
    
	bTexFlag  =  false;
	
	bMuteALL  = false;

	IsStart = false; //user did not start the game yet

	
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lightAmbient) ;

	
	//Yaw,Pitch,Roll initialization
	//yaw = 0;
	//pitch = 0;
	//roll = 0;
	initializeMovement();

	loadTargets();
	printf("targets ok\n");
	loadFragments();
	printf("fragments ok\n");
	loadNumbers();
	printf("numbers ok\n");
	loadMissile();
	printf("missile ok\n");
	loadBuilding();
	printf("building ok\n");
	//SPin angle initialization 
	mainrot_spin = 0.0;
	tailrot_spin = 0.0;
	
	//Light 2Circular angle initialization 
	light2_circular = 0.0;

	//camera position initialisation
	theta = -70;
	phi = 0;
	radius = 4.0;

	W = H = SIZE;		// global variables used to keep track of window size.

	glMatrixMode(GL_PROJECTION);
							// Set the current openGL matrix to GL_PROJECTION ie projection matrix.
	glLoadIdentity();
							// Load identitiy values in the above.

	glClearColor (1.0, 1.0, 1.0, 1.0); //make bacground color sky color
	
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);// Enable z-Buffering.
	bAxis = false;
	bWire = false;
	bSpin = false;
	bCircular = false;
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	
	/****************** NEW ************************/
	// Initialize lighting and enable color material switch so that all the glColor values are 
	//	taken in as AMBIENT AND DIFFUSE coefficients. Other material properties would be 0.
	initLights();
	 
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
	glEnable(GL_NORMALIZE);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	
	//texture:
	if(!bTexFlag)
	{
		if( !CreateTexture("./Data/Alumplat.bmp",g_Texture, 0) ||
			!CreateTexture("./Data/SnowTexture.bmp", g_Texture, 1)||
			!CreateTexture("./Data/Sky.bmp", g_Texture, 2)  ||
			!CreateTexture("./Data/base.bmp", g_Texture, 3) ||
			!CreateTexture("./Data/start.bmp", g_Texture, 4) ||
			!CreateTexture("./Data/sky_night.bmp", g_Texture, 5)||
			!CreateTexture("./Data/target.bmp", g_Texture, 6)||
			!CreateTexture("./Data/winner.bmp", g_Texture, 7)
		  )
			bTexFlag = false;	// Load the texture into memory. Remember you can only provide BMP files here.
		else bTexFlag = true;
	}

	//sounds
	g_Sound[0].filename = "./Data/battle_start.wav";
	g_Sound[0].duration = 12000;
	g_Sound[0].mute		= false;

	g_Sound[1].filename = "./Data/blakhawk.wav";
	g_Sound[1].duration = 5000;
	g_Sound[1].mute		= false;

	g_Sound[2].filename = "./Data/choppergun.wav";
	g_Sound[2].duration = 0.00;
	g_Sound[2].mute		= false;

	g_Sound[3].filename = "./Data/wind.wav";
	g_Sound[3].duration = 7000;
	g_Sound[3].mute		= false;

	g_Sound[4].filename = "./Data/G_Rooster_01.wav";
	g_Sound[4].duration = 1000;
	g_Sound[4].mute		= false;

	g_Sound[5].filename = "./Data/Frog01.wav";
	g_Sound[5].duration = 1000;
	g_Sound[5].mute		= false;
	
	g_Sound[6].filename = "./Data/explosion.wav";
	g_Sound[6].duration = 2000;
	g_Sound[6].mute		= false;
	
		/***********************************************/

	showMenu();
	BattleSound(10); //start the battle sound before user start egine.
}
/****************************************************************************************/

void Motion(void) //set action taken when image is idle no interaction from user.
{

	if(bSpin)
	{	
		tailrot_spin+=60;
		mainrot_spin += 75;
        
		if(IsStart && (movement.y < Flight_Latitude))
		{
			
            movement.y +=0.05;
		}
		else if(IsStart) //chopper ready to shoot
        {
			IsStart=false;
			//TargLevel -= 0.05;
		}	

	
		Timer(GetTickCount());	
	}

        

		


	if(bCircular)
	{
		light2_circular += 2.0;
	}
	
	glutPostRedisplay();
}			
/************************************************/
void keyboardCallbackProc(unsigned char key, int x, int y)
						// This is the callback procedure for capturing OpenGL Keyboard events.
{
	float fx, fy, fz;
	Movement temp=movement;
	switch(key)
	{

	case 'a':				// Show axes.
	case 'A':
		if (bAxis == true)			bAxis = false;			
		else						bAxis = true;		
		break;
	case 'z': //zoom in
		fovy -= 0.5;
		fViewVol -= 0.2;
		break;
	case 'Z'://zoom out
		fovy += 0.5;
		fViewVol += 0.2;
		break;
	case 'w':
	case 'W':
		if (bWire == false)			
		{
			bWire = true;	// Wireframe model.
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		}
		else
		{
			bWire = false;	// Solid model.
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		}
		break;

	case 'N':				//Change to perspective projections	
	case 'n':
		Game_env.projection = PERSPECTIVE;
		break;

	case 'O':				//Change to orthographic projections	
	case 'o':
		Game_env.projection = ORTHOGRAPHIC;
		break;

	case 'F': //move camera position forward
	case 'f':
		   //if(abs(radius - 0.1) > 0.0)	
		   //{
			   radius -= 0.1;
		   //}
		break;

	case 'b': //move camera position backward
	case 'B':
		radius += 0.1;	
		break;
	case 'c':
	case 'C':
		if(Game_env.camera == FIRST_PERSON)
		{
			fovy = FIRSTVIEWFOVY;
			fViewVol = FIRSTFVIEWVOL;
		}else if(Game_env.camera == THIRD_PERSON)
		{

			fovy = FOVY;
			fViewVol=FVIEWVOL;
		}
		
		break;
	case 's' :
	case 'S' :
		if(IsStart == 0)
		{
			IsStart = !IsStart;
			Game_env.timeday=GetTickCount(); //start timer 
		}

		bSpin = !bSpin;
		RotorSound(10);
		break;
    case 'L' :
	case 'l' :
		bMuteALL = !bMuteALL;
		PlaySound(NULL,NULL,NULL); //stip any sound being played
		RotorSound(10);
		break;
	case 'M' :
	case 'm' :
		g_Sound[1].mute= !g_Sound[1].mute;
		RotorSound(10);
		break;
	case 'X' :
	case 'x' :
		showMenu();
		break;
	case 'p':
	//	pitch += 0.1;
		//pitch has some boundary limit
		if (movement.pitch>-45)
		{
			temp.pitch-=AngleOffset;
			if (boundaryCheck(0, 0, 0, temp))
			{
				movement.pitch-=AngleOffset;
			}
		}
		break;
	case 'P':
		if (movement.pitch<45)
		{
			temp.pitch+=AngleOffset;
			if (boundaryCheck(0, 0, 0, temp))
			{
				movement.pitch+=AngleOffset;
			}
		}
	//	pitch -= 0.1; 
		break;
	case 'y':
		//yaw += 0.1; 
		//for yaw, there should be no boundary
	//	if (movement.yaw<90)
		//if (movement.yaw<90)
		{
			temp.yaw+=AngleOffset;
			if (boundaryCheck(0, 0, 0, temp))
			{
				movement.yaw+=AngleOffset;
				
				if (movement.yaw>360)
				{
					movement.yaw-=360;
				}
				
			}
		}
		break;
	case 'Y':
		//if (movement.yaw>-90)
		{
			temp.yaw-=AngleOffset;
			if (boundaryCheck(0, 0, 0, temp))
			{
				movement.yaw-=AngleOffset;
				
				if (movement.yaw<-360)
				{
					movement.yaw+=360;
				}
				
			}
		}
		//yaw -= 0.1; 
		break;
	case 'r':
		if (movement.roll>-45)
		{
			movement.roll-=AngleOffset;
		}
	//	roll += 0.1; 
		break;
	case 'R':
		if (movement.roll<45)
		{
			movement.roll+=AngleOffset;
		}
	//	roll -= 0.1; 
		break;

	case '1' :
		Game_env.camera = FIRST_PERSON; //switch to first person view
		fovy = FIRSTVIEWFOVY;
		fViewVol = FIRSTFVIEWVOL;
		PlaySound(NULL, NULL, NULL); //kill sound directly
		break;
	case '3' :
		Game_env.camera = THIRD_PERSON; //switch to third person view
		fovy = FOVY;
		fViewVol = FVIEWVOL;
		RotorSound(10);
		break;
	case 'i':
	case'I':
		theta -= AngleOffset;
		break;
	case 'k':
	case'K':
		theta += AngleOffset;
		break;
	case 'j':
	case'J':
		phi -= AngleOffset;
		break;
	case 't':
	case'T':
		phi += AngleOffset;
		break;
	/****************** NEW ************************/
	case 'h':
		lightone_direction[1]= -0.5;
		lightone_direction[2]= 1.5;
		fSpotLightone = 20.0;
		fExpLightone =10.0;
		initLightOne();
		break;

	case 'H':
		lightone_direction[1]= -2.0;
		lightone_direction[2]= 10.0;
		fSpotLightone = 30.0;
		fExpLightone =2.0;
		initLightOne();
		break;

	case '9':
		theta+=1;
		break;
	case '8':
		theta-=1;
		break;
	case '7':
		phi+=1;
		break;
	case '6':
		phi-=1;
		break;
	case '2':
		bCircular = !bCircular;
		//light2_circular += 5.0;
		break;

	/******************************************/
	case 'U':
	case 'u':		
		temp.pitch+=90;
		calcCoord(PositionOffset, temp, fx, fy, fz);
		if (boundaryCheck(fx, fy, fz))
		{
			movement.x+=fx;
			movement.y+=fy;
			movement.z+=fz;
		}	
		break;
	case 'D':
	case 'd':
		temp.pitch-=90;
		calcCoord(PositionOffset, temp, fx, fy, fz);
		if (boundaryCheck(fx, fy, fz))
		{
			movement.x+=fx;
			movement.y+=fy;
			movement.z+=fz;
		}	
		break;
	case '4':
		
		break;

	case 27 :				//ESCAPE Code for exiting program.
		exit(0);
	}
	glutPostRedisplay();
}

/****************************************************************************************/

void specialCallbackProc (int key, int x, int y)
						// This is the callback procedure for capturing special charater events.
{
	float fx, fy, fz;
	Movement temp=movement;
	switch (key)		
	{		
		case GLUT_KEY_LEFT:		// Rotate center of projection left
			//phi -= 0.5;
			temp.yaw+=90;
			calcCoord(PositionOffset, temp, fx, fy, fz);
			if (boundaryCheck(fx, fy, fz))
			{
				movement.x+=fx;
				movement.y+=fy;
				movement.z+=fz;
			}
			break;

		case GLUT_KEY_RIGHT:	// Rotate center of projection right
			temp.yaw-=90;
			calcCoord(PositionOffset, temp, fx, fy, fz);
			if (boundaryCheck(fx, fy, fz))
			{
				movement.x+=fx;
				movement.y+=fy;
				movement.z+=fz;
			}
			break;

		case GLUT_KEY_UP :		// Rotate center of projection up
			//theta -= 0.5;
			calcCoord(MovementOffset, movement, fx, fy, fz);
			if (boundaryCheck(fx, fy, fz))
			{
				movement.x+=fx;
				movement.y+=fy;
				movement.z+=fz;
			}
			//movement.z+=1;
			break;

		case GLUT_KEY_DOWN :	// Rotate center of projection down
			//theta += 0.5;
			//movement.z-=1;
			calcCoord(-MovementOffset, movement, fx, fy, fz);
			if (boundaryCheck(fx, fy, fz))
			{
				movement.x+=fx;
				movement.y+=fy;
				movement.z+=fz;
			}
			break;
		case GLUT_KEY_F1:
			bLight1 = !bLight1;
			if(!bLight1)
				glDisable(GL_LIGHT0);
			else glEnable(GL_LIGHT0);
			break;
		case GLUT_KEY_F2:
			bLight2 = !bLight2;
			if(!bLight2)
				glDisable(GL_LIGHT1);
			else glEnable(GL_LIGHT1);
			break;
		case GLUT_KEY_F3:
			bLight3 = !bLight3;
			if(!bLight3)
				glDisable(GL_LIGHT2);
			else glEnable(GL_LIGHT2);
			break;

		case GLUT_KEY_F4:
			Game_env.IsShiny = !Game_env.IsShiny;
			shinyLight(); //set on/off shiny/rusty
			
			if(Game_env.day == DAY) // PUT A FOGGY/SHINY DAY 
			{
				if(Game_env.weather == SHINY_DAY)
				{
					Game_env.weather = FOGY_DAY;
					Game_env.amdif   = LOW;
				}
                else 
				{
					Game_env.weather = SHINY_DAY;
					Game_env.amdif   = HIGH;
				}		

			}
			break;
	}
	glutPostRedisplay();
}

void Mouse(int button, int state, int x, int y)
{
	if(button == GLUT_LEFT_BUTTON)
    {
		if(state == GLUT_DOWN)
        {
			if(Game_env.camera == FIRST_PERSON && !g_Sound[2].mute && !bMuteALL)
			{
				PlaySound(g_Sound[2].filename, NULL, SND_ASYNC);
				openFire(GetTickCount(), x, y);
			}
		}
	}
}

	
/****************************************************************************************/
void reShapeCallbackProc(int w, int h)
							// This is the callback procedure for capturing reShape event for window resizing.
{
	glViewport(0, 0, w, h);
					// Set the viewport to current window size.

	W = w;  //keep track of window resizing
	H = h;

	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	if(Game_env.projection == PERSPECTIVE)
		gluPerspective(fovy, (GLfloat)W/(GLfloat)H, NEAR_Z, FAR_Z);
	else if(Game_env.projection == ORTHOGRAPHIC)
	{
		if (w > h)
		{
			W = (fViewVol * w) / h;
			H = fViewVol;
		}
		else
		{
			W = fViewVol;
			H = (fViewVol * h) / w;
		}
		glOrtho(-W, W, -H, H, NEAR_Z, FAR_Z);
	}			// Change the view volume to maintain the aspect ratio wrt to viewport.

	glMatrixMode(GL_MODELVIEW);	

	
}


/****************************************************************************************/
void displayCallbackProc (void)
						// This is the callback procedure for capturing OpenGL Display events.
{
	Movement temp=movement;
	DWORD time_elapsed;
	// Process Application Loop
	GLfloat light1_pos[4];
	GLfloat light2_pos[4];
	tickCount = GetTickCount ();				// Get The Tick Count
	time_elapsed=tickCount-Game_env.lastTickCount;
	Update (tickCount - Game_env.lastTickCount);	// Update The Counter
	Game_env.lastTickCount = tickCount;		

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
							// Set the current openGL matrix to GL_PROJECTION ie projection matrix.
	glLoadIdentity();
							// Load identitiy values in the above.
	if(Game_env.projection == PERSPECTIVE)
	{	
		
		gluPerspective(fovy, (GLfloat)W/(GLfloat)H, NEAR_Z, FAR_Z);
							// This sets the view volume to be a Perspective one.
	}
	else if(Game_env.projection == ORTHOGRAPHIC)
	{
	
		glOrtho(-fViewVol, fViewVol, -fViewVol, fViewVol, NEAR_Z, FAR_Z);
							// This sets the view volume to be a Orthographic one.
	}
	glMatrixMode(GL_MODELVIEW);
	glColor3f(0.0, 0.0, 0.0);


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	

	if (targetCount==0)
	{
		Game_env.camera=THIRD_PERSON;
		initializeMovement();
		//movement.pitch=movement.roll=movement.yaw=0;
		//movement.x=0;
		//movement.z=2;
		//movement.y=HecLevel;
	
	}
		
	//glLightfv(GL_LIGHT2 ,GL_SPOT_DIRECTION, lightthr_direction);
	//glLightfv(GL_LIGHT2, GL_POSITION, lightthr_position);
	
	//IN POLAR COORDINATES
	if(Game_env.camera == THIRD_PERSON)
	{	
		glutSetCursor(GLUT_CURSOR_NONE);
		GLfloat fEye_x = radius * sin(theta*PI/180.0) * sin(phi*PI/180.0)+movement.x;
		GLfloat fEye_y = radius * cos(theta*PI/180.0)+movement.y;
		GLfloat fEye_z = radius * sin(theta*PI/180.0) * cos(phi*PI/180.0)+movement.z;
		
		//gluLookAt(fEye_x, fEye_y, fEye_z, movement.x, movement.y, 1.0, 0, 1, 0);	
		gluLookAt(fEye_x, fEye_y, fEye_z, movement.x, movement.y, movement.z, 0, 1, 0);	
		// Specify the same camera position and orientation using polar coordiantes.

		glPushMatrix();
			//glTranslatef(-fEye_x, -fEye_y, -fEye_z);
			showReferenceAxis();
			drawEnvironment();
			drawBuilding();
		//drawTestPlan();
			
			drawRockets(tickCount);
			//target must be drawn AFTER rocket because explosion might happen
			drawTargets(time_elapsed);
			drawExplosion(tickCount);
		//glPushMatrix();
			glTranslatef(movement.x, movement.y, movement.z);
			//glTranslatef(-fEye_x, -fEye_y, -fEye_z);
			glRotatef(movement.yaw , 0.0, 1.0, 0.0);
			glRotatef(-movement.pitch , 1.0, 0.0, 0.0);
			glRotatef(movement.roll , 0.0, 0.0, 1.0);
			//glTranslatef(movement.x, movement.y, movement.z);
			
			drawChopper();
			if (targetCount==0)
			{
				glTranslatef(0, 2, 2);
				drawAnnounce(7);
			}
			//drawNumber(time_elapsed);
			//drawNumber(12345);
			//glTranslatef(fEye_x, fEye_y, fEye_z);
		glPopMatrix();
	}
	else if (Game_env.camera == FIRST_PERSON)
	{
        //glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR );

		glutSetCursor(GLUT_CURSOR_CYCLE);
		 
		//First person view

		GLfloat fLook_x=movement.x+
			sin(deg2rad(movement.yaw))*cos(deg2rad(movement.pitch));
		GLfloat fLook_y=movement.y+
			sin(deg2rad(movement.pitch))*cos(deg2rad(movement.roll)); 
		GLfloat fLook_z=movement.z+
			cos(deg2rad(movement.yaw))*cos(deg2rad(movement.pitch));

		GLfloat fUp_x = sin(deg2rad(movement.roll));
		GLfloat fUp_y = cos(deg2rad(movement.roll));
		GLfloat fUp_z = 0;


		gluLookAt(movement.x, movement.y, movement.z, fLook_x, fLook_y, fLook_z, 
			fUp_x, fUp_y, fUp_z);
		

		showReferenceAxis();
	
		
		drawEnvironment();
		drawBuilding();
		drawRockets(tickCount);
	//drawTestPlan();
////////////////////////////////////////////////////////////////////////////////////////////
		//target must be drawn AFTER rocket because explosion might happen
		drawTargets(time_elapsed);
		drawExplosion(tickCount);

		glPushMatrix();
			glTranslatef(movement.x, movement.y, movement.z);
			glRotatef(movement.yaw , 0.0, 1.0, 0.0);
			glRotatef(-movement.pitch , 1.0, 0.0, 0.0);
			glRotatef(movement.roll , 0.0, 0.0, 1.0);
			//glTranslatef(movement.x, movement.y, movement.z);
			
			drawChopper();
		glPopMatrix();
		//drawNumber(time_elapsed);
		//drawNumber(12345);
	}
	

	
	/************* NEW *****************/
	// Lights Positioning:

	//light 1:	
	light1_pos[0]=lightone_position[0]+movement.x;
	light1_pos[1]=lightone_position[1]+movement.y;
	light1_pos[2]=lightone_position[2]+movement.z;
	light1_pos[3]=1;


	light2_pos[0]=lighttwo_position[0]+movement.x;
	light2_pos[1]=lighttwo_position[1]+movement.y;
	light2_pos[2]=lighttwo_position[2]+movement.z;
	light2_pos[3]=1;
		

	temp.pitch+=LightOnePitchOffset;
	calcCoord(1, temp, lightone_direction[0], lightone_direction[1], lightone_direction[2]);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, lightone_direction);
	glLightfv(GL_LIGHT0, GL_POSITION, light1_pos);
	
	//light 2:
	//glPushMatrix();
		//glRotatef(light2_circular, 0.0,1.0, 0.0);
	glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, lighttwo_direction);
	glLightfv(GL_LIGHT1, GL_POSITION, light2_pos);
	//glPopMatrix();
	/***********************************/
    
	/********************* New *************/
    


	if(!bSpin && !IsStart)
    {
	   drawAnnounce(4);
	}

		


	glutSwapBuffers();
}

/****************************************************************************************/
int main (int argc, char *argv[])
						// The main program.
{
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(SIZE, SIZE);
	glutInitWindowPosition(300, 200);
	glutCreateWindow("City Chopper 2005."); //create the window with title 

	// Ask The User If They Want To Start In FullScreen Mode?
   if (MessageBox (HWND_DESKTOP, "Would You Like To Run In Fullscreen Mode?", "Start FullScreen?", MB_YESNO | MB_ICONQUESTION) == IDYES)
   {
	 glutFullScreen  ( ); 	// If yes, Run In fullScreen Mode
   }
	

	init();				// Initialize the env. variables and the program global variables.

	
	/* Callback registrations with the OpenGL env are done here */
	glutDisplayFunc(displayCallbackProc);  //call dispaly function call
	glutKeyboardFunc(keyboardCallbackProc); // call keyboard event handler function
	glutSpecialFunc(specialCallbackProc); //call keyboard special keys event handler
	glutReshapeFunc(reShapeCallbackProc);  // call reshape image function
	glutIdleFunc(Motion);//call idle handle function
	glutMouseFunc( Mouse );//call mouse event
	
	/* Callback registrations done.*/

	
	glutMainLoop(); // loop until kill request
	return 1;
}



void loadTargets()
{
	int i;
	for (i=0; i<TargetFileCount; i++)
	{
		targetModelLists[i]=glmReadOBJ(targetFileNameLists[i]);
		glmUnitize(targetModelLists[i]);					//Translate the model to origin and scale to fit in a unit cube.
		glmFacetNormals(targetModelLists[i]);				//Generate face normals. 
		glmVertexNormals(targetModelLists[i], 90.0);	
	}
	targetCount=MaxTargetCount;
	for (i=0; i<MaxTargetCount; i++)
	{
		targets[i].coordIndex=i;
		//the first group are tanks
		if (i<6)
		{
			targets[i].modelIndex=0;
		}
		else
		{
			//these are plane
			targets[i].modelIndex=1;
		}
	}
}



void drawTargets(DWORD time_elapsed)
{
	glEnable(GL_TEXTURE_2D);
		//glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D, g_Texture[6]);
	
	GLfloat x, y, z;
	int coord_index, model_index;
	for (int i=0; i<targetCount; i++)
	{
		glPushMatrix();
		coord_index=targets[i].coordIndex;
		model_index=targets[i].modelIndex;
		//update target position so they are moving.
		//update first
		if (targetCoordLists[coord_index][2]>-FlyBoundary)
		{
			targetCoordLists[coord_index][2]-=TargetMovementSpeed*time_elapsed;
		}
		else
		{
			targetCoordLists[coord_index][2]+=FlyBoundary-10;
		}

		x=targetCoordLists[coord_index][0];
		y=targetCoordLists[coord_index][1];
		z=targetCoordLists[coord_index][2];
		glTranslatef(x,y,z);
		glRotatef(180, 0,1,0);
		glmDraw(targetModelLists[model_index], GLM_SMOOTH | GLM_TEXTURE);
		
	
		glPopMatrix();

	}

	glDisable(GL_TEXTURE_2D);
}


void openFire(DWORD now, int x, int y)
{
	float ratio, h_angle, v_angle;
	if (rocketCount<MaxRocketCount)
	{
		v_angle=fovy/2.0;
		h_angle=v_angle*(float)W/(float)H;

		rockets[rocketCount].fireMoment=now;

		//rockets[rocketCount].firePos=movement;
		rockets[rocketCount].firePos.x=movement.x;
		rockets[rocketCount].firePos.y =movement.y;
		rockets[rocketCount].firePos.z=movement.z;
		rockets[rocketCount].firePos.pitch=movement.pitch;
		rockets[rocketCount].firePos.yaw=movement.yaw;
		rockets[rocketCount].firePos.roll=movement.roll;

		ratio=(float)(H/2.0-y)/(H/2.0);
		rockets[rocketCount].firePos.pitch+= ratio*v_angle;
		ratio=(float)(W/2.0-x)/(W/2.0);

		rockets[rocketCount].firePos.yaw+=ratio*h_angle;

		rocketCount++;
	}
}

void drawRockets(DWORD now)
{
	float x, y, z;
	

	glPushMatrix();

	for (int i=0; i<rocketCount; i++)
	{
		if (calcRocket(i, now, x, y, z))
		{		
			//glLoadIdentity();
			glPushMatrix();
			//glTranslatef(-movement.x, -movement.y, -movement.z);
			glTranslatef(rockets[i].firePos.x, rockets[i].firePos.y, rockets[i].firePos.z);
			//glTranslatef((movement.x+rockets[i].firePos.x)/2.0, 
			//	(FLOOR_LEVEL+rockets[i].firePos.y)/2.0, (movement.z+rockets[i].firePos.z)/2.0);

			glTranslatef(x, y, z);
			drawRocket();
			glPopMatrix();
		}
		//check if there is explosion happened
		if (explosionIndex!=-1)
		{
			//draw explosion at the coord of target and then remove target
			//the explosionIndex is only the index of a list of target
			createExplosion(targetCoordLists[targets[explosionIndex].coordIndex], now);
			targetCount--;
			for (int j=explosionIndex; j<targetCount; j++)
			{
				targets[j]=targets[j+1];
			}
			explosionIndex=-1;
		}

	}
	
	//glLoadIdentity();
	//glTranslatef(-movement.x, -movement.y, -movement.z);
	
	//glTranslatef(0, FLOOR_LEVEL+1, 12);
	//drawRocket();

	glPopMatrix();
}


void drawRocket()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glPushMatrix();
	glScalef(0.1, 0.1, 0.1);
	//glTranslatef(movement.x, movement.y, movement.z);
	glRotatef(movement.yaw , 0.0, 1.0, 0.0);
	glRotatef(-movement.pitch , 1.0, 0.0, 0.0);
	glRotatef(movement.roll , 0.0, 0.0, 1.0);
	glColor3f(1,0,0);
	glmDraw(missileModelLists[0], GLM_SMOOTH | GLM_TEXTURE);
	//glColor3f(1,0,0);
	//glutSolidSphere(RocketSize, 20, 20);
	//glutSolidSphere(0.1, 20, 20);
	glPopMatrix();
	glPopAttrib();
}

bool boundaryCheck(float fx, float fy, float fz, const Movement& mv)
{
	float safeHeight;
	/*
	if (!(abs(fx)<FlyBoundary&&abs(fz)<FlyBoundary
				&&fy>MinLatitude&&fy<MaxLatitude))
	{
		return false;
	}
	safeHeight=ChopperLength*cos(deg2rad(mv.roll))*sin(deg2rad(mv.pitch));
	//pitch>0 means head up tail down and we must not let chopper tail touch ground
	if (mv.pitch>0&&fy<safeHeight+MinLatitude)
	{
		return false;
	}
	return true;
	*/
	
	if (!(abs(mv.x+fx)<FlyBoundary&&abs(mv.z+fz)<FlyBoundary
				&&mv.y+fy>MinLatitude&&mv.y+fy<MaxLatitude))
	{
		return false;
	}
	safeHeight=ChopperLength*cos(deg2rad(mv.roll))*sin(deg2rad(mv.pitch));
	//pitch>0 means head up tail down and we must not let chopper tail touch ground
	if (mv.pitch>0&&mv.y+fy<safeHeight+MinLatitude)
	{
		return false;
	}
	return true;
	
}

void createExplosion(float coord[3], DWORD now)
{
	glPushMatrix();
	glTranslatef(-movement.x, -movement.y, -movement.z);
	glTranslatef(coord[0], coord[1], coord[2]);
	//explosionCount=rand()%(MaxExplosionPieces-1)+1;//at least one
	explosionCount=MaxFragmentCount;//at least one
	for (int i=0; i<explosionCount; i++)
	{
		explosions[i].firePos.x=coord[0];
		explosions[i].firePos.y=coord[1];
		explosions[i].firePos.z=coord[2];
		explosions[i].firePos.pitch=rand()%90*(rand()%2==0?(-1):1);
		explosions[i].firePos.yaw=rand()%360;
		explosions[i].firePos.roll=rand()%90*(rand()%2==0?(-1):1);
		explosions[i].fireMoment=now;
	}
}

void drawExplosion(DWORD now)
{
	float distance, interval;
	float x, y, z;
	if (explosionCount>0)
	{
		interval=now-explosions[0].fireMoment;

		if (interval>MaxFragmentFlyTime)
		{
			explosionCount=0;
			return;
		}
		distance=interval*FragmentFlySpeed;
		for (int i=0; i<explosionCount; i++)
		{
			calcCoord(distance, explosions[i].firePos, x, y, z);
			glPushAttrib(GL_ALL_ATTRIB_BITS);
			glPushMatrix();
			glTranslatef(explosions[i].firePos.x, 
				explosions[i].firePos.y, explosions[i].firePos.z);
			glTranslatef(x, y, z);
			drawExplosionPiece(i);
			glPopMatrix();
			glPopAttrib();
		}
	}
}

void drawExplosionPiece(int index)
{
	float r,g,b, size;
	r=rand()%11*0.1;
	g=rand()%11*0.1;
	b=rand()%11*0.1;
	size=rand()%10*0.1;
	glColor3f(r,g,b);
	glutSolidCube(size);
	//glmDraw(fragmentModelLists[index], GLM_SMOOTH | GLM_TEXTURE);
}

void loadFragments()
{
	char buf[256];
	char* ptr;
	strcpy(buf, "./data/frag_0.obj");
	ptr=strchr(buf, '0');
	for (int i=0; i<MaxFragmentCount; i++)
	{
		*ptr=i+'1';
		fragmentModelLists[i]=glmReadOBJ(buf);
		glmUnitize(fragmentModelLists[i]);					//Translate the model to origin and scale to fit in a unit cube.
		glmFacetNormals(fragmentModelLists[i]);				//Generate face normals. 
		glmVertexNormals(fragmentModelLists[i], 90.0);
	}
}

void loadNumbers()
{
	char buf[256];
	char* ptr;
	strcpy(buf, "./data/numbers_0.obj");
	ptr=strchr(buf, '0');
	for (int i=0; i<10; i++)
	{		
		*ptr=i+'0';
		numberModelLists[i]=glmReadOBJ(buf);
		glmUnitize(numberModelLists[i]);					//Translate the model to origin and scale to fit in a unit cube.
		glmFacetNormals(numberModelLists[i]);				//Generate face normals. 
		glmVertexNormals(numberModelLists[i], 90.0);
	}
}

void loadMissile()
{
	for (int i=0; i<MaxMissileModelCount; i++)
	{
		missileModelLists[i]=glmReadOBJ("./data/missile.obj");
		glmUnitize(missileModelLists[i]);					//Translate the model to origin and scale to fit in a unit cube.
		glmFacetNormals(missileModelLists[i]);				//Generate face normals. 
		glmVertexNormals(missileModelLists[i], 90.0);
	}
}

void drawBuilding()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glPushMatrix();
	glEnable(GL_TEXTURE_2D);
		//glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D, g_Texture[1]);
	
	//glScalef(0.1, 0.1, 0.1);
	for (int i=0; i<MaxSingleBuildingCount; i++)
	{
		glTranslatef(singleBuildingCoordLists[i][0], singleBuildingCoordLists[i][1],
			singleBuildingCoordLists[i][2]);
		glScalef(5,5,5);

		glmDraw(singleBuildingModel, GLM_SMOOTH | GLM_TEXTURE);
	}
	//glColor3f(1,0,0);
	//glutSolidSphere(RocketSize, 20, 20);
	//glutSolidSphere(0.1, 20, 20);
	glDisable(GL_TEXTURE_2D);

	glPopMatrix();
	glPopAttrib();	
	
	/*
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glPushMatrix();
	//glScalef(0.1, 0.1, 0.1);
	glTranslatef(manyBuildingCoord[0], manyBuildingCoord[1], manyBuildingCoord[2]);

	glmDraw(manyBuildingModel, GLM_SMOOTH | GLM_TEXTURE);
	//glColor3f(1,0,0);
	//glutSolidSphere(RocketSize, 20, 20);
	//glutSolidSphere(0.1, 20, 20);
	glPopMatrix();
	glPopAttrib();	
	*/
}


void loadBuilding()
{
	singleBuildingModel=glmReadOBJ(singleBuildingFileName);
	glmUnitize(singleBuildingModel);					//Translate the model to origin and scale to fit in a unit cube.
	glmFacetNormals(singleBuildingModel);				//Generate face normals. 
	glmVertexNormals(singleBuildingModel, 90.0);
	
	for (int i=0; i<MaxSingleBuildingCount; i++)
	{
		singleBuildingCoordLists[i][0]=rand()%100*0.5*(rand()%2==0?1:(-1));
		singleBuildingCoordLists[i][1]=FLOOR_LEVEL;
		singleBuildingCoordLists[i][0]=rand()%100*0.5*(rand()%2==0?1:(-1));
	}


	/*
	manyBuildingModel=glmReadOBJ(manyBuildingFileName);
	glmUnitize(manyBuildingModel);					//Translate the model to origin and scale to fit in a unit cube.
	glmFacetNormals(manyBuildingModel);				//Generate face normals. 
	glmVertexNormals(manyBuildingModel, 90.0);
	*/
}

void drawNumber(int number)
{
	int mod, coff=number;
	int pos=0;
	do
	{
		mod=coff%10;
		drawDigit(mod, pos, 0);
		pos++;
		coff=coff/10;
	}
	while (coff!=0);
}

void drawDigit(int num, int pos, int start)
{
	
//	float x,y,z;
	Movement temp=movement;
	glPushMatrix();

	/*
	temp.yaw-=(start+pos)*DigitDisplayAngleOffset;

	calcCoord(DigitDisplayPositionOffset, temp, x, y, z);

	glTranslatef(x,y,z);
	glTranslatef(temp.x, temp.y, temp.z);
	glRotatef(temp.yaw+180, 0.0, 1.0, 0.0);
	glRotatef(-temp.pitch , 1.0, 0.0, 0.0);
	glRotatef(temp.roll , 0.0, 0.0, 1.0);
	glmDraw(numberModelLists[num], GLM_SMOOTH | GLM_TEXTURE);
		*/

	glTranslatef(0, FLOOR_LEVEL, 0);
	glTranslatef(-temp.x, -temp.y, -temp.z);
	
	glScalef(10, 10, 10);
	//glmDraw(numberModelLists[num], GLM_SMOOTH | GLM_TEXTURE);
	glmDraw(singleBuildingModel, GLM_SMOOTH | GLM_TEXTURE);
	glPopMatrix();
	
	

}


/*************************** END OF PROGRAM ************************************************/